/**
 * @brief database update script, target version V1.02.03
 *
 * @author Norbert Koch
 * @date   13.11.2015
 *
 * Copyright (c) by Bachmann Technology GmbH & Co. KG
 */

.bail ON

-- Add missing item types for a slave.
-- In version V1.01.xx only the pdu id was used to distinguish a master from a slave.
-- This is no longer true, because we now may have power units.  Therefore, we need
-- an item type.  Fill up the entries.

INSERT OR REPLACE INTO ItemTypeDefinition
SELECT sensorvariableid|70, 3 FROM SensorVariableDef
WHERE  pduid > 0 AND sensortype = 0 AND sensorid_byte0 = 255 AND keyid = 0;

-- connect a gpio output (a gpo) to a signal chain, i.e.
-- if an alarm is related to the signal chain some lamp/horn you name it is triggered
-- values for the gpo are defined in this table
-- valuetype: ON, OFF, CYCLE_ON, CYCLE_OFF,
-- cycletime: if a cycle is defined set the target state (on/off) for n seconds (0 = forever)
-- repetitions: repeat the above action n times (0 = forever, defaut one)
-- cycleinterval: successive repetitions happen every n seconds (0 = forever)
CREATE TABLE SignalChainHasGPO (
		signalchainid   INTEGER NOT NULL,
		gpoid           INTEGER NOT NULL,
		valuetype       INTEGER NOT NULL,
		cycletime       INTEGER NOT NULL DEFAULT 0,
		repetitions     INTEGER NOT NULL DEFAULT 1,
		cycleinterval   INTEGER NOT NULL DEFAULT 0,
		PRIMARY KEY (signalchainid, gpoid),
		FOREIGN KEY (signalchainid) REFERENCES SignalChain (signalchainid)
									ON UPDATE CASCADE
									ON DELETE CASCADE,
		FOREIGN KEY (gpoid)         REFERENCES SensorVariableDef (sensorvariableid)
									ON UPDATE CASCADE
									ON DELETE CASCADE);


-- i2c gpio sensor configuration
-- defines the mode of each input and output (HIGH, LOW, DISABLED, S0 (pulse))
-- as well as the factor used to calculate a real value from the number of pulses (the pulse constant)
CREATE TABLE GPIOConfiguration (
	   sensorvariableid  INTEGER      NOT NULL UNIQUE,
	   o0mode            INTEGER      NOT NULL,
	   o1mode            INTEGER      NOT NULL,
	   o2mode            INTEGER      NOT NULL,
	   o3mode            INTEGER      NOT NULL,
	   i0mode            INTEGER      NOT NULL,
	   i1mode            INTEGER      NOT NULL,
	   i2mode            INTEGER      NOT NULL,
	   i3mode            INTEGER      NOT NULL,
	   i0pc              INTEGER      NOT NULL,
	   i1pc              INTEGER      NOT NULL,
	   i2pc              INTEGER      NOT NULL,
	   i3pc              INTEGER      NOT NULL,
	   FOREIGN KEY (sensorvariableid) REFERENCES SensorVariableDef (sensorvariableid)
									  ON UPDATE RESTRICT
									  ON DELETE CASCADE);

-- new i2c related eventlog entries

INSERT INTO EventLogModule (id, mname) VALUES (14, 'GPIO');

INSERT INTO EventLogModuleAction (mid, aid, active, format) VALUES
(14,  1, 1, 'Configuration for sensor @SVID@ has been added: @MSG@'),
(14,  3, 1, 'Configuration for sensor @SVID@ has been modified: @MSG@'),
(14,  2, 1, 'Configuration for sensor @SVID@ has been deleted');

-- the rcm self-test event-log entry has been reworded

UPDATE EventLogModuleAction
SET format = 'RCM selftest @RCMRESULT@ for @RCMNAME@ triggered on @PDU@ @SN@, measured I(AC)=@RCMACVAL@ mA, I(DC)=@RCMDCVAL@ mA: AC alarm @RCMACDET@ and DC alarm @RCMDCDET@ (applied current > 50 mA)'
WHERE mid = 13 AND aid = 15;

-- entries for general warning and error

INSERT INTO EventLogAction (id, aname) VALUES
(19, 'warning');

INSERT INTO EventLogModuleAction (mid, aid, active, format) VALUES
(12, 19, 1, '@MSG@'),
(12, 13, 1, '@MSG@');

-- authentication server changes
INSERT INTO EventLogModuleAction (mid, aid, active, format) VALUES
( 1,  1, 1, '@SERVICE@ server @ID@ has been created: @KVP@'),
( 1,  2, 1, '@SERVICE@ server @ID@ has been deleted: @KVP@'),
( 1,  3, 1, '@SERVICE@ server @ID@ has been updated: @KVP@');


-- the scaling of energy values has been changed
-- from -4 to -1.  Adapt the database

UPDATE SetpointScaleUnit
SET    scaling = -1
WHERE  sensorvariableid IN (
	   SELECT sensorvariableid
	   FROM   sensorvariabledef
	   WHERE  keyid IN (32, 33, 34, 35, 36, 37, 38));

-- Simply drop the old table, as it was not in use yet!
DROP TABLE RelayState;
-- Create the new table, which allows the storage of the identify
CREATE TABLE RelayState (
	   sensorVariableID	INTEGER NOT NULL PRIMARY KEY,
	   switchState		INTEGER NOT NULL DEFAULT 1 CHECK (switchState    BETWEEN 0 AND 1),
	   identifyState	INTEGER NOT NULL DEFAULT 0 CHECK (identifyState  BETWEEN 0 AND 1));


-- set the home directory for user admin
UPDATE User SET userhome = '/home/admin' WHERE userid = 1000;

-- upcase the language if it still has its default setting
UPDATE User SET userlanguage = 'DE' WHERE userid = 1000 AND userlanguage = 'de';

-- make sure user admin is also member of the operator group
INSERT OR IGNORE INTO UserHasUserRole (userId, roleId) VALUES (1000, 1001);


-- generalize the relation between signal chains and e-mail recipients
-- because we allow externally defined users as well

CREATE TABLE SignalChainToEmailReceiverTmp (
		signalchainid  INTEGER NOT NULL,
		rectype		   INTEGER NOT NULL DEFAULT 1 CHECK (rectype BETWEEN 1 AND 5),
		userid         INTEGER,
		username       TEXT,
		FOREIGN KEY (signalchainid) REFERENCES SignalChain (signalchainid)
									ON UPDATE RESTRICT
									ON DELETE CASCADE
		FOREIGN KEY (userid)        REFERENCES User (userid)
									ON UPDATE RESTRICT
									ON DELETE CASCADE);

INSERT INTO SignalChainToEmailReceiverTmp (signalchainid, userid)
SELECT signalchainid, userid FROM SignalChainToEmailReceiver;

PRAGMA foreign_keys = OFF;

DROP TABLE SignalChainToEmailReceiver;
ALTER TABLE SignalChainToEmailReceiverTmp RENAME TO SignalChainToEmailReceiver;

PRAGMA foreign_keys_check;
PRAGMA foreign_keys = ON;


CREATE TRIGGER UniqueSC2EROnInsert BEFORE INSERT ON SignalChainToEmailReceiver
FOR EACH ROW
	WHEN (new.userid IS NOT NULL
		  AND 0 < (SELECT COUNT(signalchainid) FROM SignalChainToEmailReceiver
				   WHERE signalchainid = new.signalchainid
				   AND   rectype       = new.rectype
				   AND   userid        = new.userid))
   OR  (new.username IS NOT NULL
		  AND 0 < (SELECT COUNT(signalchainid) FROM SignalChainToEmailReceiver
				   WHERE signalchainid = new.signalchainid
				   AND   rectype       = new.rectype
				   AND   username      = new.username))
BEGIN
	SELECT RAISE(ABORT, 'UNIQUE constraint failed: SignalChainToEmailReceiver');
END;


CREATE TRIGGER UniqueSC2EROnUpdate BEFORE UPDATE ON SignalChainToEmailReceiver
FOR EACH ROW
	WHEN (new.userid IS NOT NULL
		  AND 0 < (SELECT COUNT(signalchainid) FROM SignalChainToEmailReceiver
				   WHERE signalchainid = new.signalchainid
				   AND   rectype       = new.rectype
				   AND   userid        = new.userid))
   OR  (new.username IS NOT NULL
		  AND 0 < (SELECT COUNT(signalchainid) FROM SignalChainToEmailReceiver
				   WHERE signalchainid = new.signalchainid
				   AND   rectype       = new.rectype
				   AND   username      = new.username))
BEGIN
	SELECT RAISE(ABORT, 'UNIQUE constraint failed: SignalChainToEmailReceiver');
END;

--- remove the master build number from the stringvalue table
DELETE FROM StringValue WHERE sensorvariableid = 0x0000ffffffff020a;


--- enter external authentication servers
/**
 * The superset of all authentication server data fields we need to have.
 * At the time of writing, this is only LDAP and AD (server type 2).
 * RADIUS and TACACS+ are to follow on (BTG) user request
 *
 * - serverID        internally used uniquely server identification
 * - service         one of the authentication services we support (LDAP, RADIUS, TACACS+)
 * - enabled         the server is active or not (can be queried)
 * - orderNumber     gives the order in which a servers of one service are queried
 *                   one after the other, e.g. for a user's e-mail address.
 *                   The search stops on the first match.
 * - authType        RADIUS: authentication protocol, CHAP, PAP
 *                   LDAP:   server type (hence the protocol) OPEN_LDAP (SASL), MS_AD (KERBEROS)
 * - retries         number of retries to connect the server
 * - connTimeout     time in seconds we wait for the server to answer on a connection request
 * - searchTimeout   time in seconds we wait for the server to answer on a search request
 *
 * The table already existed in 1.01.xx but unused.
 * We have to drop it first.
 */
DROP TABLE IF EXISTS AuthenticationServer;
CREATE TABLE AuthenticationServer (
	id                    INTEGER NOT NULL PRIMARY KEY,
	service               INTEGER NOT NULL CHECK (service BETWEEN 0 AND 2),
	enabled               INTEGER NOT NULL,
	orderNumber           INTEGER NOT NULL,
	serverAddress         TEXT NOT NULL,
	serverPort            INTEGER NOT NULL CHECK (serverPort BETWEEN 0 AND 65535),
	authType              INTEGER NOT NULL CHECK (authType BETWEEN 0 AND 3),
	retries               INTEGER NOT NULL CHECK (retries BETWEEN 1 AND 100),
	connTimeout           INTEGER NOT NULL CHECK (connTimeout BETWEEN 1 AND 500),
	searchTimeout         INTEGER NOT NULL CHECK (searchTimeout BETWEEN 1 AND 500),
	userName              TEXT NOT NULL,
	passphrase            TEXT NOT NULL,
	baseDN                TEXT NOT NULL,
	baseNamingAttr        TEXT NOT NULL,
	groupDN               TEXT NOT NULL,
	groupNamingAttr       TEXT NOT NULL,
	userLoginDN           TEXT NOT NULL,
	userObjectClass		  TEXT NOT NULL,
	userSearchFilter	  TEXT NOT NULL,
	UNIQUE (service, serverAddress, serverPort)
);

-- an event log action for the update
INSERT INTO EventLogAction (id, aname) VALUES (18, 'update');


-- element name change for pdus and slaves (c.f. BNPROD-2520)
UPDATE SensorName
SET    variablename = 'Master'
WHERE  sensorvariableid = (
	   SELECT sensorvariableid
	   FROM SensorVariableDef
	   WHERE pduid = 0 AND sensortype = 0 AND sensorid_byte0 = 255 AND keyid = 0);

UPDATE SensorName
SET    variablename = 'Slave-' || (sensorvariableid >> 56)
WHERE  sensorvariableid IN (
	   SELECT sensorvariableid
	   FROM SensorVariableDef
	   WHERE pduid > 0 AND sensortype = 0 AND sensorid_byte0 = 255 AND keyid = 0);


-- define the home directory for all exisiting users
-- by default (this is what we set) /home/${LOGIN}

UPDATE User SET userhome = '/home/' || LOWER(username);

-- Remove old guid value definitions, which have been appended by accident (BNPROD-2433)
-- New entries do not deliver this anymore. It suffices to get rid of them
DELETE FROM SetpointScaleUnit WHERE sensorvariableid IN (
	SELECT sensorvariableid FROM SensorVariableDef WHERE pduid != 0 AND keyid = 0
);


-- Change the scaling of the slaves
-- BNPROD-2684 - Change the frequency
UPDATE setpointscaleunit SET scaling=-1 WHERE sensorVariableID IN (SELECT sensorVariableID FROM SensorVariableDef WHERE keyID = 23);
-- BNPROD-2684 - Change the power scales
UPDATE setpointscaleunit SET scaling=-1 WHERE sensorVariableID IN (SELECT sensorVariableID FROM SensorVariableDef WHERE keyID = 18 or keyID = 19 or keyID = 20 or keyID = 22);

-- target version
PRAGMA user_version = 10203;
