/**
 * @brief database update script, target version V2.00.00
 *
 * @author Martin Güthle
 * @date   14.06.2016
 *
 * Copyright (c) by Bachmann Technology GmbH & Co. KG
 */

.bail ON

-- ----------------------------------------------------------------------------
-- BNPROD-2716
-- add new log entries for hardware error and warning (ignore existing)

INSERT OR IGNORE INTO EventLogModuleAction (mid, aid, active, format) VALUES
( 9, 13, 1, '@MSG@'),
( 9, 19, 1, '@MSG@');

-- update rcm selftest result string
UPDATE EventLogModuleAction
SET format = 'RCM selftest @RCMRESULT@ for @RCMNAME@ triggered on @SN@, measured I(AC)=@RCMACVAL@ mA, I(DC)=@RCMDCVAL@ mA: AC alarm @RCMACDET@ and DC alarm @RCMDCDET@ (applied current > 50 mA)'
WHERE mid = 13 AND aid = 15;

-- ----------------------------------------------------------------------------
-- BNPROD-2878
-- make some setpoints only partly changeable

-- Power Factor: lower boundaries only
UPDATE SetpointScaleUnit
SET    setpointtype = 10
WHERE  sensorvariableid IN (SELECT sensorvariableid FROM SensorVariableDef WHERE keyid = 17);

-- RCM: upper boundaries only
UPDATE SetpointScaleUnit
SET    setpointtype = 5
WHERE  sensorvariableid IN (SELECT sensorvariableid FROM SensorVariableDef WHERE keyid IN (7, 8));

-- ----------------------------------------------------------------------------
-- BNPROD-2040
-- the user shall have the ability to set the alarm type of a GPIO input state change
-- to this end we add a column which overwrites the alarmtype with a default of do nothing

ALTER TABLE AlarmToSignalchain
ADD COLUMN scalarm INTEGER NOT NULL DEFAULT -1;


-- ----------------------------------------------------------------------------
-- store socket groups which have to be re-created after the slave factory reset is completed

CREATE TABLE SlaveSocketGroupReappend (
	   socketID	INTEGER NOT NULL,
	   groupID	INTEGER NOT NULL,
	   PRIMARY KEY(socketID, groupID),
	   FOREIGN KEY (groupID) REFERENCES SensorVariableDef (sensorVariableID)
									  ON UPDATE RESTRICT
									  ON DELETE CASCADE);

-- ----------------------------------------------------------------------------
-- BNPROD-2952
-- the fuse shall be only present, if there is a real fuse
-- therefor get ride of any existing ones
PRAGMA foreign_keys = ON;

-- Divide the sensorvariableID, to get ride of the actual used key (consits of 2 bytes => /256/256)
DELETE FROM ItemTypeDefinition WHERE sensorVariableID/256/256 IN (
	SELECT sensorVariableID/256/256 FROM SensorVariableDef WHERE pduID = 0 AND sensorID_byte3 = 255 AND sensorID_byte2 != 255
	AND ((
		select count(*) FROM SensorVariableDef WHERE pduID = 0 AND sensorID_byte3 = 255 AND sensorID_byte2 != 255 AND sensorID_byte1 == 0 AND keyID = 0
	) == 1)
);

DELETE FROM SensorVariableDef WHERE sensorVariableID IN (
	SELECT sensorVariableID FROM SensorVariableDef WHERE pduID = 0 AND sensorID_byte3 = 255 AND sensorID_byte2 != 255
	AND ((
		select count(*) FROM SensorVariableDef WHERE pduID = 0 AND sensorID_byte3 = 255 AND sensorID_byte2 != 255 AND sensorID_byte1 == 0 AND keyID = 0
	) == 1)
);

-- ----------------------------------------------------------------------------
-- BNPROD-3123
-- the RCM type has to be set to BTG_BN2_RCM_TYPE_RCM (= 1)
-- Limit this to all values, which are 0, even if this should not be relevant for the moment
UPDATE ItemTypeDefinition set definition = 1 WHERE sensorVariableID & 0xFFFF = 0x0047 AND definition = 0;
UPDATE ItemTypeDefinition set definition = 1 WHERE sensorVariableID & 0xFFFF = 0x0045 AND definition = 0;

-- ----------------------------------------------------------------------------
-- BNPROD-3103
-- Support eleven slaves, i.e. twelve devices
-- This requires an update in sensorvariabledef and knownslaves due to constraints

PRAGMA foreign_keys = OFF;

CREATE TABLE SensorVariableDefTmp (
	   sensorVariableID	  INTEGER NOT NULL PRIMARY KEY,
	   pduID			  INTEGER NOT NULL CHECK (pduID BETWEEN 0 AND 11 OR pduID = 255),
	   sensorType		  INTEGER NOT NULL CHECK (sensorType BETWEEN 0 AND 4),
	   sensorID_byte0	  INTEGER NOT NULL,
	   sensorID_byte1	  INTEGER NOT NULL,
	   sensorID_byte2	  INTEGER NOT NULL,
	   sensorID_byte3	  INTEGER NOT NULL,
	   keyID			  INTEGER NOT NULL);

INSERT INTO SensorVariableDefTmp SELECT * FROM SensorVariableDef;

DROP TABLE  SensorVariableDef;
ALTER TABLE SensorVariableDefTmp RENAME TO SensorVariableDef;

PRAGMA foreign_keys_check;
PRAGMA foreign_keys = ON;


CREATE TABLE KnownSlavesTmp (
	articleNumber         TEXT NOT NULL,
	serialNumber          TEXT NOT NULL,
	pduID                 INTEGER NOT NULL CHECK (pduID BETWEEN 1 AND 11),
	PRIMARY KEY (pduID)
);

INSERT INTO KnownSlavesTmp SELECT * FROM KnownSlaves;

DROP TABLE  KnownSlaves;
ALTER TABLE KnownSlavesTmp RENAME TO KnownSlaves;

-- ----------------------------------------------------------------------------
-- BNPROD-1324
-- Set groups to SPV_ALL (15) which have at least one monitored socket attached
-- Set all other groups to SPV_NONE (0)
-- The group entries in question have sensor type 2 and key id 4 (CURRENT)
-- The connection table Socket Group uses key id 0 (GUID) => +/- 4

UPDATE SetpointScaleUnit
SET    setpointtype =
CASE
  WHEN (
    SELECT    COUNT(ssu.sensorvariableid)
    FROM      SocketGroup sg
    LEFT JOIN SetpointScaleUnit ssu
    ON        ssu.sensorvariableid = sg.socketid + 4
    WHERE     ssu.setpointtype = 15
    AND       sg.groupid = SetpointScaleUnit.sensorvariableid - 4) > 0
  THEN  15
  ELSE  0
END
WHERE sensorvariableid IN (
  SELECT sensorvariableid
  FROM   SensorVariableDef
  WHERE  sensortype = 2
  AND    keyid = 4);

-- ---------------------------------------------------------------------------
-- BNPROD-3348
-- Move snmp location and contact from the StringValue table to Configuration
-- If the entries don't exist, create empty ones
--
-- It's possible the original entries do not exist (rare cases).  Therefore,
-- we create the entries with empty string to ease our access.
--
-- If someone has succeeded in creating entries with a NULL value, we also
-- translate this into an empty string

PRAGMA foreign_keys = ON;

INSERT OR IGNORE INTO SensorVariableDef VALUES
(0xffffffff0200, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x0200),
(0xffffffff0201, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x0201);

INSERT OR IGNORE INTO StringValue VALUES
(0xffffffff0200, ''), (0xffffffff0201, '');

INSERT OR IGNORE INTO Configuration (itemkey, itemtype, itemvalstring)
VALUES
('SnmpDeviceLocation', 1, (SELECT IFNULL(stringvalue, '')
					   	   FROM StringValue
						   WHERE sensorvariableid = 0xffffffff0200)),
('SnmpDeviceContact', 1, (SELECT IFNULL(stringvalue, '')
					  	  FROM StringValue
						  WHERE sensorvariableid = 0xffffffff0201));

DELETE FROM SensorVariableDef
WHERE sensorvariableid IN (0xffffffff0200, 0xffffffff0201);

-- ---------------------------------------------------------------------------
-- target version
PRAGMA user_version = 20000;
